@@ -18,7 +18,7 @@ use Sub::Exporter -setup =>
 	};
 	
 use vars qw ($VERSION);
-$VERSION     = '0.03';
+$VERSION     = '0.04';
 }
 
 #-------------------------------------------------------------------------------
@@ -72,14 +72,18 @@ to create an easy to understand dump of binary data. This achieved through:
 
 =head1 DOCUMENTATION
 
-This module was inspired by the B<hexd> command from libma L<http://www.ioplex.com/~miallen/libmba/>.
+The shortest perl dumper is C<perl -ne 'BEGIN{$/=\16} printf "%07x0: @{[unpack q{(H2)*}]}\n", $.-1'>, coutesy of a golfing session 
+with Andrew Rodland <arodland@cpan.org> aka I<hobbs> on #perl.
 
-Binary data is split according to user defined I<ranges> and rendered as a B<hex> or/and B<decimal> data dump.
+B<hexd> from libma L<http://www.ioplex.com/~miallen/libmba/> is nice tools that inspired me to writr this module. It may be a better 
+alternative If you need speed when generating dumps.
+
+B<Data::HexDump::Range> splits binary data according to user defined I<ranges> and rendered as a B<hex> or/and B<decimal> data dump.
 The data dump can be rendered in ANSI, ASCII or HTML.
 
 =head2 Orientation
 
-The examples below show the hypothetic ranges:
+The examples below show the hypothetic ranges below applied to the source code of this module:
 
   my $data_range = # definition to re-use
 	[
@@ -105,8 +109,6 @@ The examples below show the hypothetic ranges:
 	  ],
 	] ;
 	
-Applied to the source code of this module.
-
 =head3 Vertical
 
 In this orientation mode, each range displayed separately starting with the range name
@@ -195,7 +197,7 @@ The color definition is one of:
  
 =item * An RGB color definition - eg: todo: add example
 
-=item * undef - will be repaced by a white color or pickec from a cyclic color list (see B<COLOR> in L<new>).
+=item * undef - will be repaced by a white color or picked from a cyclic color list (see B<COLOR> in L<new>).
 
 =back
 
@@ -261,7 +263,7 @@ a subroutine definition.
   my $dynamic_range =
 	[
 	  [\&name, \&size, \&color ],
-	  [\&define_range] # returns a sub range definition
+	  [\&define_range] # returns a range definition
 	] ;
 
 =head4 'name' sub ref
@@ -294,11 +296,58 @@ a subroutine definition.
   
   $hdr->dump(['data', 100, \&alternate_color], $data) ;
 
-=head4 Range definition defined by a subroutine reference
+=head4  User defined range generator
 
-  my $hdr = Data::HexDump::Range->new() ;
+A subroutine reference can be passed as a range definition. The cubroutine will be called repetitively
+till the data is exhausted or the subroutine returns I<undef>.
+
+  sub my_parser 
+  	{
+  	my ($data, $offset) = @_ ;
+  	
+  	my $first_byte = unpack ("x$offset C", $data) ;
+  	
+  	$offset < length($data)
+  		?  $first_byte == ord(0)
+  			? ['from odd', 5, 'blue on_yellow']
+  			: ['from even', 3, 'green']
+  		: undef ;
+  	}
   
-  print $hdr->dump(\&parser, $data) ;
+  my $hdr = Data::HexDump::Range->new() ;
+  print $hdr->dump(\&my_parser, '01' x 50) ;
+
+=head2 user_defined_parser($data, $offset)
+
+Add information, according to the options passed to the constructor, to the internal data.
+
+I<Arguments> - See L<gather>
+
+=over 2
+
+=item * $data - Binary string - the data passed to the I<dump> method
+
+=item * $offset - Integer - current offset in $data
+
+=back
+
+I<Returns> - 
+
+=over 2
+
+=item * $range - An array reference containing a name, size and color
+
+OR
+
+=item * undef - Done parsing
+
+=back
+
+=cut
+
+=head1 EXAMPLES
+
+See L<HDR.html>
 
 =head1 OTHER IDEAS
 
@@ -326,11 +375,13 @@ Readonly my $NEW_ARGUMENTS =>
 	COLOR 
 	OFFSET_FORMAT 
 	DATA_WIDTH 
+	DISPLAY_COLUMN_NAMES
 	DISPLAY_OFFSET DISPLAY_CUMULATIVE_OFFSET
 	DISPLAY_ZERO_SIZE_RANGE_WARNING
 	DISPLAY_ZERO_SIZE_RANGE 
 	DISPLAY_RANGE_NAME
 	MAXIMUM_RANGE_NAME_SIZE
+	DISPLAY_RANGE_SIZE
 	DISPLAY_ASCII_DUMP
 	DISPLAY_HEX_DUMP
 	DISPLAY_DEC_DUMP 
@@ -355,10 +406,12 @@ Create a Data::HexDump::Range object.
 		DATA_WIDTH => 16 | 20 | ... ,
 		DISPLAY_RANGE_NAME => 1 ,
 		MAXIMUM_RANGE_NAME_SIZE => 16,
+		DISPLAY_COLUMN_NAMES => 0,
 		DISPLAY_OFFSET  => 1 ,
 		DISPLAY_CUMULATIVE_OFFSET  => 1 ,
 		DISPLAY_ZERO_SIZE_RANGE_WARNING => 1,
 		DISPLAY_ZERO_SIZE_RANGE => 1,
+		DISPLAY_RANGE_SIZE => 1,
 		DISPLAY_ASCII_DUMP => 1 ,
 		DISPLAY_HEX_DUMP => 1,
 		DISPLAY_DEC_DUMP => 1,
@@ -366,9 +419,7 @@ Create a Data::HexDump::Range object.
 		ORIENTATION => 'horizontal',
 		) ;
 
-I<Arguments> - A list of named arguments
-
-All arguments are optional. Settings get a default value if not passed by user. Default values are listed below.
+I<Arguments> - All arguments are optional. Default values are listed below.
 
 =over 2 
 
@@ -398,23 +449,25 @@ in base 10. Default is 'hex'.
 
 =item * DISPLAY_RANGE_NAME - Boolean - If set, range names are displayed in the dump.
 
-=item * MAXIMUM_RANGE_NAME_SIZE - Integer - maximum size of a range name (in horizontal mode)
+=item * MAXIMUM_RANGE_NAME_SIZE - Integer - maximum size of a range name (horizontal mode). Default size is 16.
 
-Default size is 16.
+=item * DISPLAY_COLUMN_NAMES - Boolean -  If set, the column names are displayed. Default I<false>
 
-=item * DISPLAY_OFFSET - Boolean - If set, the offset columnis displayed in the dump.
+=item * DISPLAY_OFFSET - Boolean - If set, the offset column is displayed. Default I<true>
 
-=item * DISPLAY_CUMULATIVE_OFFSET - Boolean - If set, the cumulative offset column is displayed in 'vertical' rendering mode
+=item * DISPLAY_CUMULATIVE_OFFSET - Boolean - If set, the cumulative offset column is displayed in 'vertical' rendering mode. Default is I<true>
 
 =item * DISPLAY_ZERO_SIZE_RANGE - Boolean - if set, ranges that do not consume data are displayed. default is I<true> 
 
-=item * DISPLAY_ZERO_SIZE_RANGE_WARNING - Boolean - if set, a warning is emitted if ranges that do not consume data. default is I<true> 
+=item * DISPLAY_ZERO_SIZE_RANGE_WARNING - Boolean - if set, a warning is emitted if ranges that do not consume data. Default is I<true> 
 
-=item * DISPLAY_ASCII_DUMP - Boolean - If set, the ASCII representation of the binary data is displayed
+=item * DISPLAY_RANGE_SIZE - Bolean - if set the range size is prepended to the name. Default I<false>
 
-=item * DISPLAY_HEX_DUMP - Boolean - If set, the hexadecimal dump column is displayed
+=item * DISPLAY_ASCII_DUMP - Boolean - If set, the ASCII representation of the binary data is displayed. Default is I<true>
 
-=item * DISPLAY_DEC_DUMP - Boolean - If set, the decimall dump column is displayed
+=item * DISPLAY_HEX_DUMP - Boolean - If set, the hexadecimal dump column is displayed. Default is I<true>
+
+=item * DISPLAY_DEC_DUMP - Boolean - If set, the decimall dump column is displayed. Default is I<false>
 
 =item * COLOR_NAMES - A hash reference
 
@@ -508,6 +561,9 @@ $self->CheckOptionNames($NEW_ARGUMENTS, @setup_data) ;
 	
 	DISPLAY_RANGE_NAME => 1,
 	MAXIMUM_RANGE_NAME_SIZE => 16,
+	DISPLAY_RANGE_SIZE => 1,
+	
+	DISPLAY_COLUMN_NAMES  => 0 ,
 	DISPLAY_OFFSET => 1,
 	DISPLAY_CUMULATIVE_OFFSET => 1,
 	DISPLAY_HEX_DUMP => 1,
@@ -532,7 +588,10 @@ if($self->{VERBOSE})
 $self->{OFFSET_FORMAT} = $self->{OFFSET_FORMAT} =~ /^hex/ ? "%08x" : "%010d" ;
 $self->{MAXIMUM_RANGE_NAME_SIZE} = 2 if$self->{MAXIMUM_RANGE_NAME_SIZE} <= 2 ;
 
-#todo: check all the options values
+$self->{FIELDS_TO_DISPLAY} =  $self->{ORIENTATION} =~ /^hor/
+	? [qw(OFFSET HEX_DUMP DEC_DUMP ASCII_DUMP RANGE_NAME)]
+	: [qw(RANGE_NAME OFFSET CUMULATIVE_OFFSET HEX_DUMP DEC_DUMP ASCII_DUMP)] ;
+
 
 return(1) ;
 }
@@ -610,7 +669,7 @@ I<Arguments>
 
 =over 2 
 
-=item * $range_description - See L<Range definiton>
+=item * $range_description - See L<Range definition>
   
 =item * $data - A string - binary data to dump
 
@@ -671,12 +730,15 @@ I<Exceptions> - None
 
 my ($self) = @_ ;
 
-return $self->format($self->split($self->{GATHERED})) ;
+my $split_data = $self->split($self->{GATHERED}) ;
+
+$self->add_information($split_data) ;
+
+return $self->format($split_data) ;
 }
 
 #-------------------------------------------------------------------------------
 
-
 sub dump
 {
 
@@ -692,13 +754,73 @@ I<Exceptions> - dies if the range description is invalid
 
 =cut
 
-my ($self) = shift ;
+my ($self, $range_description, $data, $offset, $size) = @_ ;
 
 return unless defined wantarray ;
 
-my ($gathered_data, $used_data) = $self->_gather(undef, @_) ;
+my ($gathered_data, $used_data) = $self->_gather(undef, $range_description, $data, $offset, $size) ;
 
-return $self->format($self->split($gathered_data)) ;
+my $split_data = $self->split($gathered_data) ;
+
+$self->add_information($split_data) ;
+
+return $self->format($split_data) ;
+}
+
+#-------------------------------------------------------------------------------
+
+sub add_information
+{
+
+=head2 [P] add_information($split_data)
+
+Add information, according to the options passed to the constructor, to the internal data.
+
+I<Arguments> - See L<gather>
+
+=over 2
+
+=item * $split_data - data returned by _gather()
+
+=back
+
+I<Returns> - Nothing
+
+I<Exceptions> - None
+
+=cut
+
+my ($self, $split_data) = @_ ;
+
+if($self->{DISPLAY_COLUMN_NAMES})
+	{
+	my $information = '' ;
+	
+	for my $field_name (@{$self->{FIELDS_TO_DISPLAY}})
+		{
+		if(exists $split_data->[0]{$field_name})
+			{
+			my $length = 0 ;
+			
+			for (@{$split_data->[0]{$field_name}})
+				{
+				$length += length($_->{$field_name}) ;
+				}
+				
+			$information .= sprintf "%-${length}.${length}s ", $field_name
+			}
+		else
+			{
+			$information .= ' ' ;
+			}
+		}
+		
+	unshift @{$split_data},
+		{
+		INFORMATION => [ {INFORMATION => ' ' . $information} ], 
+		NEW_LINE => 1,
+		} ;
+	}
 }
 
 #-------------------------------------------------------------------------------
@@ -804,7 +926,25 @@ I<Exceptions> dies if passed invalid parameters
 
 my ($self, $collected_data, $range_description, $data, $offset, $size) = @_ ;
 
-my $ranges = $self->create_ranges($range_description) ;
+my $range_provider ;
+
+if('CODE' eq ref($range_description))
+	{
+	$range_provider = $range_description ;
+	}
+else
+	{
+	my $ranges = $self->create_ranges($range_description) ;
+	
+	$range_provider = 
+		sub
+		{
+		while(@{$ranges})
+			{
+			return shift @{$ranges} ;
+			}
+		}
+	}
 
 my $used_data = $offset || 0 ;
 
@@ -819,11 +959,14 @@ $size = defined $size ? min($size, length($data) - $used_data) : length($data) -
 my $location = "$self->{FILE}:$self->{LINE}" ;
 my $skip_ranges = 0 ;
 
-for my $range (@{$ranges})
+while(my $range  = $range_provider->($data, $used_data))
 	{
 	my ($range_name, $range_size, $range_color) = @{$range} ;
 	my $is_comment = 0 ;
 	
+	#~ use Data::TreeDumper ;
+	#~ print DumpTree $range ;
+	
 	if('' eq ref($range_size))
 		{
 		if('#' eq  $range_size)
@@ -848,6 +991,11 @@ for my $range (@{$ranges})
 	
 	($range_name, $range_size, $range_color) = @sub_or_scalar ;
 	
+	if(!$is_comment && $self->{DISPLAY_RANGE_SIZE})
+		{
+		$range_name = $range_size . ':' . $range_name ;
+		}
+	
 	$self->{INTERACTION}{WARN}("Warning: range '$range_name' requires zero bytes.\n")
 		if(!$is_comment && $range_size == 0 && $self->{DISPLAY_ZERO_SIZE_RANGE_WARNING}) ;
 		
@@ -1336,10 +1484,8 @@ for ($self->{FORMAT})
 	
 		my $colorizer = /ASCII/ ? sub {$_[0]} : \&colored ;
 		
-		my @fields = 
-			$self->{ORIENTATION} =~ /^hor/
-				? qw(OFFSET HEX_DUMP DEC_DUMP ASCII_DUMP RANGE_NAME)
-				: qw( RANGE_NAME OFFSET CUMULATIVE_OFFSET HEX_DUMP DEC_DUMP ASCII_DUMP) ;
+		my @fields = @{$self->{FIELDS_TO_DISPLAY}} ;
+		unshift @fields, 'INFORMATION' ;
 
 		for my $line (@{$line_data})
 			{
@@ -1435,6 +1581,6 @@ L<http://search.cpan.org/dist/Data-HexDump-Range>
 
 =head1 SEE ALSO
 
-L<Data::Hexdumper>
+L<Data::Hexdumper>, L<Data::ParseBinary>, L<Convert::Binary::C>, L<Parse::Binary>
 
 =cut
